@@ -87,12 +87,17 @@ $Config{'numeric'} = 0;
 $Config{'pathtocat'} = "cat";
 $Config{'pathtozcat'} = "zcat";
 $Config{'pathtobzcat'} = "bzcat";
+$Config{'pathtoxzcat'} = "xzcat";
 $Config{'output'} = "stdout"; #8.0
 $Config{'format'} = "text"; #8.0
 $Config{'encode'} = "none"; #8.0
 $Config{'hostformat'} = "none"; #8.0
 $Config{'html_wrap'} = 80;
-$Config{'supress_ignores'} = 0;
+$Config{'suppress_ignores'} = 0;
+$Config{'hostlimit'} = "";
+$Config{'appendvaradmtologdirs'} = 1;
+$Config{'appendvarlogtologdirs'} = 1;
+$Config{'appendcwdtologdirs'} = 1;
 
 if (-e "$ConfigDir/conf/html/header.html") {
    $Config{'html_header'} = "$ConfigDir/conf/html/header.html";
@@ -143,7 +148,7 @@ if ($Config{'debug'} > 8) {
 # For each of the configuration sets (logwatch.conf here, and
 # logfiles,and services later), we do the following:
 #  1. read the different configuration files
-#  2. for each parameter, if it is cummulative, check if
+#  2. for each parameter, if it is cumulative, check if
 #     it the special case empty string
 #  3. check to see if duplicate
 
@@ -174,6 +179,11 @@ for (my $i = 0; $i <= $#ReadConfigNames; $i++) {
    }
 }
 
+my @LogDirs=("$Config{'logdir'}/");
+push @LogDirs, "/var/adm/" if $Config{'appendvaradmtologdirs'};
+push @LogDirs, "/var/log/" if $Config{'appendvarlogtologdirs'};
+push @LogDirs, "" if $Config{'appendcwdtologdirs'};
+
 &CleanVars();
 
 if ($Config{'debug'} > 8) {
@@ -433,7 +443,7 @@ for $ThisFile (@logfiles) {
                   @{$LogFileData{$ThisLogFile}{'logfiles'}} = ();
                } else {
                   if ($ReadConfigValues[$i] !~ m=^/=) {
-                     foreach my $dir ("$Config{'logdir'}/", "/var/adm/", "/var/log/", "") {
+                     foreach my $dir (@LogDirs) {
                         # We glob to obtain filenames.  We reverse in case
                         # we use the decimal suffix (.0, .1, etc.) in filenames
                         #@TempLogFileList = reverse(glob($dir . $ReadConfigValues[$i]));
@@ -472,7 +482,7 @@ for $ThisFile (@logfiles) {
                   @{$LogFileData{$ThisLogFile}{'archives'}} = ();
                } else {
                   if ($ReadConfigValues[$i] !~ m=^/=) {
-                     foreach my $dir ("$Config{'logdir'}/", "/var/adm/", "/var/log/", "") {
+                     foreach my $dir (@LogDirs) {
                         # We glob to obtain filenames.  We reverse in case
                         # we use the decimal suffix (.0, .1, etc.) in filenames
                         #@TempLogFileList = reverse(glob($dir . $ReadConfigValues[$i]));
@@ -725,7 +735,16 @@ foreach $LogFile (@LogFileList) {
 	}
 
    @FileList = $TempDir . $LogFile . "-archive";
-   push @FileList, @{$LogFileData{$LogFile}{'logfiles'}};
+
+   # Handle compressed log files using the archive codepath
+   foreach my $lf (@{$LogFileData{$LogFile}{'logfiles'}}) {
+      if ($lf =~ /\.(?:gz|bz2|xz)$/) {
+         push @{$LogFileData{$LogFile}{'archives'}}, $lf;
+      } else {
+         push @FileList, $lf;
+      }
+   }
+
    my $DestFile =  $TempDir . $LogFile . "-archive";
    my $Archive;
    foreach $Archive (@{$LogFileData{$LogFile}{'archives'}}) {
@@ -767,6 +786,10 @@ foreach $LogFile (@LogFileList) {
             my $arguments = "'${Archive}' 2>/dev/null >> $DestFile";
             system("$Config{'pathtobzcat'} $arguments") == 0
                or die "system '$Config{'pathtobzcat'} $arguments' failed: $?"
+         } elsif (($Archive =~ m/xz$/) && (-f "$Archive") && (-s "$Archive")) {
+            my $arguments = "'${Archive}' 2>/dev/null >> $DestFile";
+            system("$Config{'pathtoxzcat'} $arguments") == 0
+               or die "system '$Config{'pathtoxzcat'} $arguments' failed: $?"
          } elsif ((-f "$Archive") && (-s "$Archive")) {
             my $arguments = "'${Archive}'  >> $DestFile";
             system("$Config{'pathtocat'} $arguments") == 0
@@ -778,8 +801,8 @@ foreach $LogFile (@LogFileList) {
    my $FileText = "";
 
    foreach my $ThisFile (@FileList) {
-      #Existence check for files -mgt
-      next unless (-f $ThisFile);
+      #Existence check for files and character devices such as /dev/null
+      next unless (-f $ThisFile || -c $ThisFile );
       if ($ThisFile =~ /'/) {
          print "File $ThisFile has invalid embedded quotes.  File ignored.\n";
 	 next;
@@ -803,6 +826,17 @@ foreach $LogFile (@LogFileList) {
    @EnvList = ();
 
    my $FilterText = " ";
+   if (-e "/usr/bin/iconv") {
+      if (defined $Config{'charencoding'}) {
+         my $from_encoding;
+         if ($Config{'charencoding'}) {
+            $from_encoding = "-f $Config{'charencoding'}";
+         } else {
+            $from_encoding = "";
+         }
+         $FilterText = " | iconv -c $from_encoding -t UTF-8 - ";
+      }
+   }
    foreach (sort keys %{$LogFileData{$LogFile}}) {
       my $cmd = $_;
       if ($cmd =~ s/^\d+-\*//) {
@@ -1080,7 +1114,7 @@ sub ReadConfigFile {
 sub Usage () {
    # Show usage for this program
    print "\nUsage: $0 [--detail <level>] [--logfile <name>] [--output <output_type>]\n" .
-      "   [--format <format_type>] [--encode <enconding>] [--numeric]\n" .
+      "   [--format <format_type>] [--encode <encoding>] [--numeric]\n" .
       "   [--mailto <addr>] [--archives] [--range <range>] [--debug <level>]\n" .
       "   [--filename <filename>] [--help|--usage] [--version] [--service <name>]\n" .
       "   [--hostformat <host_format type>] [--hostlimit <host1,host2>] [--html_wrap <num_characters>]\n\n";
@@ -1090,7 +1124,7 @@ sub Usage () {
    print "--service <name>: *Name of a service definition to report on.\n";
    print "--output <output type>: Report Output - stdout [default], mail, file.\n"; #8.0
    print "--format <formatting>: Report Format - text [default], html.\n"; #8.0
-   print "--encode <encoding>: Enconding to use - none [default], base64.\n"; #8.0
+   print "--encode <encoding>: Encoding to use - none [default], base64.\n"; #8.0
    print "--mailto <addr>: Mail report to <addr>.\n";
    print "--archives: Use archived log files too.\n";
    print "--filename <filename>: Used to specify they filename to save to. --filename <filename> [Forces output to file].\n";
@@ -1161,9 +1195,9 @@ sub initprint {
          }
          #Config{output} html
          if ( $Config{'format'} eq "html" ) {
-            $out_mime .= "Content-Type: text/html; charset=\"iso-8859-1\"\n\n";
+            $out_mime .= "Content-Type: text/html; charset=\"UTF-8\"\n\n";
          } else {
-            $out_mime .= "Content-Type: text/plain; charset=\"iso-8859-1\"\n\n";
+            $out_mime .= "Content-Type: text/plain; charset=\"UTF-8\"\n\n";
          }
 
          if ($Config{'hostformat'} eq "split") { #8.0 check hostlimit also? or ne none?
@@ -1399,10 +1433,10 @@ sub parselogs {
 
          if ($has_output and $ServiceData{$Service}{'title'}) {
             if ( $Config{'format'} eq "html" ) {
-                if ( ($Ignored > 0) && ($Config{'supress_ignores'} == 0) ) {  &output( $index_par, "\n $Ignored Ignored Lines\n", "header"); };
+                if ( ($Ignored > 0) && ($Config{'suppress_ignores'} == 0) ) {  &output( $index_par, "\n $Ignored Ignored Lines\n", "header"); };
                 #&output( $index_par,  "\n <h3><font color=\"blue\">$ServiceData{$Service}{'title'} End </font></h3>\n", "header");
             } else {
-                if ( ($Ignored > 0) && ($Config{'supress_ignores'} == 0) ) { &output( $index_par, "\n $Ignored Ignored Lines\n", "line"); };
+                if ( ($Ignored > 0) && ($Config{'suppress_ignores'} == 0) ) { &output( $index_par, "\n $Ignored Ignored Lines\n", "line"); };
                 &output( $index_par,  "\n ---------------------- $ServiceData{$Service}{'title'} End ------------------------- \n\n", "line");
             }
             &output( $index_par, "\n", "stop");
@@ -1460,14 +1494,15 @@ sub parselogs {
    if (defined fileno OUTFILE) {
       print OUTFILE $out_mime if $out_mime;
       if ( $Config{'encode'} eq "base64" ) {
-         print OUTFILE encode_base64($out_head) if $out_head;
-         print OUTFILE encode_base64($out_reference) if $out_reference;
+         my $out = '';
+         $out .= $out_head if $out_head;
+         $out .= $out_reference if $out_reference;
          foreach ( 0 .. $index_par ) {
-            print OUTFILE encode_base64($out_body{$_}) if defined( $out_body{$_} );
-#fixme
+            $out .= $out_body{$_} if defined( $out_body{$_} );
             $out_body{$_} = ''; #We should track this down out_body could be an array instead also -mgt
          }
-         print OUTFILE encode_base64($out_foot) if $out_foot;
+         $out .= $out_foot if $out_foot;
+         print OUTFILE encode_base64($out);
       } else {
          print OUTFILE $out_head if $out_head;
          print OUTFILE $out_reference if $out_reference;
